% Routine to run TShoen/JCandy(mods) SIR code
% Tom Shoen, University of Linkoping, PhD thesis 2006
%
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%               RUN Parameters
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
Nsamps=100;                 % No. time samples
N = 10000;                  % Number of particles
P0 = 5;                     % Initial state covariance
Rww = 10;                   % Process noise covariance
Rvv = 1;                    % Measurement noise covariance
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%               Functions
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
like = inline('1/(2*pi*1)^(1/2)*exp(-(x.^2)/(2*1))');  % likelihood (Rvv=1)
a = inline('x./2+25*x./(1+x.^2)+8*cos(1.2*t)','x','t');% process model
c = inline('(x.^2)/20');                               % measurement model
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%               INITIAL Conditions
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
x(1) = sqrt(P0)*randn(1);                             % Initial state value
y(1) = feval(c,x(1)) + sqrt(Rvv)*randn(1);            % Initial measurement

%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%       SIMULATION: main time loop (generate "slices")
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

for t = 2:Nsamps % Simulate the system
    x(t) = feval(a,x(t-1),t) + sqrt(Rww)*randn(1);
    y(t) = feval(c,x(t)) + sqrt(Rvv)*randn(1);
end
% save results
xTrue = x;
yTrue=y;
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%               RUN Bootstrap Particle Filter
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

xhat = SIR(y,a,c,like,Rww,P0,N,Nsamps);                  % conditional mean

%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
%               PLOT Results
%:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

figure('NumberTitle','off','Name','PF Results','color',[.8 .8 .8],...
    'units','norm','position',[.4,.1,.55,.8]);
subplot(2,1,1)
plot(1:Nsamps,xTrue,':b',1:Nsamps,xhat,'r','LineWidth',2);
xlabel('Sample No.');
title('Estimated States and Simulated States ')
legend('Sim States', 'Est States')

yhat = feval(c,xhat);
subplot(2,1,2)
plot(1:Nsamps,yTrue,':b',1:Nsamps,yhat,'r','LineWidth',2);
xlabel('Sample No.');
title('Estimated Measurements and Simulated Measurements')
legend(' Sim. Meas.', 'Est Meas.')
